# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .parse_usage import ParseUsage
from .bounding_box import BoundingBox

__all__ = ["EditResponse", "FormSchema"]


class FormSchema(BaseModel):
    bbox: BoundingBox
    """Bounding box coordinates of the widget"""

    description: str
    """Description of the widget extracted from the document"""

    type: Literal["text", "checkbox", "radio", "dropdown", "barcode"]
    """Type of the form widget"""

    fill: Optional[bool] = None
    """If True (default), the system will attempt to fill this widget.

    If False, the widget will be created but intentionally left unfilled.
    """

    value: Optional[str] = None
    """
    If provided, this value will be used directly instead of attempting to
    intelligently determine the field value.
    """


class EditResponse(BaseModel):
    document_url: str
    """Presigned URL to download the edited document."""

    form_schema: Optional[List[FormSchema]] = None
    """Form schema for PDF forms.

    List of widgets with their types, descriptions, and bounding boxes.
    """

    usage: Optional[ParseUsage] = None
    """
    Usage information for the edit operation, including number of pages and credits
    charged.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .chunking_config import ChunkingConfig
from .table_summary_config import TableSummaryConfig
from .figure_summary_config import FigureSummaryConfig

__all__ = ["BaseProcessingOptions"]


class BaseProcessingOptions(BaseModel):
    chunking: Optional[ChunkingConfig] = None
    """The configuration options for chunking.

    Chunking is commonly used for RAG usecases.
    """

    extraction_mode: Optional[Literal["ocr", "metadata", "hybrid"]] = None
    """The mode to use for extraction.

    Metadata/hybrid are only recommended with high quality metadata embeddings.
    """

    figure_summary: Optional[FigureSummaryConfig] = None
    """The configuration options for figure summarization."""

    filter_blocks: Optional[
        List[
            Literal[
                "Header",
                "Footer",
                "Title",
                "Section Header",
                "Page Number",
                "List Item",
                "Figure",
                "Table",
                "Key Value",
                "Text",
                "Comment",
                "Signature",
            ]
        ]
    ] = None
    """A list of block types to filter from chunk content.

    Pass blocks to filter them from content. By default, no blocks are filtered.
    """

    force_url_result: Optional[bool] = None
    """
    Force the result to be returned in URL form (by default only used for very large
    responses).
    """

    ocr_mode: Optional[Literal["standard", "agentic"]] = None
    """The mode to use for OCR.

    Agentic mode adds an extra pass, correcting any table/text mistakes at a small
    cost.
    """

    table_summary: Optional[TableSummaryConfig] = None
    """The configuration options for table summarization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PageRange"]


class PageRange(TypedDict, total=False):
    end: Optional[int]
    """The page number to stop processing at (1-indexed)."""

    start: Optional[int]
    """The page number to start processing from (1-indexed)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["WebhookConfigNew"]


class WebhookConfigNew(BaseModel):
    channels: Optional[List[str]] = None
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    metadata: Optional[object] = None
    """JSON metadata included in webhook request body"""

    mode: Optional[Literal["disabled", "svix", "direct"]] = None
    """The mode to use for webhook delivery.

    Defaults to 'disabled'. We recommend using 'svix' for production environments.
    """

    url: Optional[str] = None
    """The URL to send the webhook to (if using direct webhoook)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast

import httpx

from ..types import job_get_all_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.job_get_response import JobGetResponse
from ..types.job_get_all_response import JobGetAllResponse

__all__ = ["JobResource", "AsyncJobResource"]


class JobResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JobResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return JobResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return JobResourceWithStreamingResponse(self)

    def cancel(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Cancel Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._post(
            path_template("/cancel/{job_id}", job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetResponse:
        """
        Retrieve Parse

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return cast(
            JobGetResponse,
            self._get(
                path_template("/job/{job_id}", job_id=job_id),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, JobGetResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get_all(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        exclude_configs: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetAllResponse:
        """Get Jobs

        Args:
          cursor: Cursor for pagination.

        Use the next_cursor from the previous response to fetch
              the next page.

          exclude_configs: Exclude raw_config from response to reduce size

          limit: Maximum number of jobs to return per page. Defaults to 100, max 500.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/jobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "exclude_configs": exclude_configs,
                        "limit": limit,
                    },
                    job_get_all_params.JobGetAllParams,
                ),
            ),
            cast_to=JobGetAllResponse,
        )


class AsyncJobResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncJobResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncJobResourceWithStreamingResponse(self)

    async def cancel(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Cancel Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._post(
            path_template("/cancel/{job_id}", job_id=job_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetResponse:
        """
        Retrieve Parse

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return cast(
            JobGetResponse,
            await self._get(
                path_template("/job/{job_id}", job_id=job_id),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, JobGetResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get_all(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        exclude_configs: bool | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetAllResponse:
        """Get Jobs

        Args:
          cursor: Cursor for pagination.

        Use the next_cursor from the previous response to fetch
              the next page.

          exclude_configs: Exclude raw_config from response to reduce size

          limit: Maximum number of jobs to return per page. Defaults to 100, max 500.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/jobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cursor": cursor,
                        "exclude_configs": exclude_configs,
                        "limit": limit,
                    },
                    job_get_all_params.JobGetAllParams,
                ),
            ),
            cast_to=JobGetAllResponse,
        )


class JobResourceWithRawResponse:
    def __init__(self, job: JobResource) -> None:
        self._job = job

        self.cancel = to_raw_response_wrapper(
            job.cancel,
        )
        self.get = to_raw_response_wrapper(
            job.get,
        )
        self.get_all = to_raw_response_wrapper(
            job.get_all,
        )


class AsyncJobResourceWithRawResponse:
    def __init__(self, job: AsyncJobResource) -> None:
        self._job = job

        self.cancel = async_to_raw_response_wrapper(
            job.cancel,
        )
        self.get = async_to_raw_response_wrapper(
            job.get,
        )
        self.get_all = async_to_raw_response_wrapper(
            job.get_all,
        )


class JobResourceWithStreamingResponse:
    def __init__(self, job: JobResource) -> None:
        self._job = job

        self.cancel = to_streamed_response_wrapper(
            job.cancel,
        )
        self.get = to_streamed_response_wrapper(
            job.get,
        )
        self.get_all = to_streamed_response_wrapper(
            job.get_all,
        )


class AsyncJobResourceWithStreamingResponse:
    def __init__(self, job: AsyncJobResource) -> None:
        self._job = job

        self.cancel = async_to_streamed_response_wrapper(
            job.cancel,
        )
        self.get = async_to_streamed_response_wrapper(
            job.get,
        )
        self.get_all = async_to_streamed_response_wrapper(
            job.get_all,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel
from .v3_extract import V3Extract
from .edit_response import EditResponse
from .parse_response import ParseResponse
from .split_response import SplitResponse
from .extract_response import ExtractResponse
from .classify_response import ClassifyResponse
from .pipeline_response import PipelineResponse

__all__ = [
    "JobGetResponse",
    "AsyncJobResponse",
    "AsyncJobResponseResult",
    "EnhancedAsyncJobResponse",
    "EnhancedAsyncJobResponseResult",
]

AsyncJobResponseResult: TypeAlias = Union[
    ParseResponse, ExtractResponse, SplitResponse, EditResponse, PipelineResponse, V3Extract, ClassifyResponse, None
]


class AsyncJobResponse(BaseModel):
    status: Literal["Pending", "Completed", "Failed", "Idle"]

    progress: Optional[float] = None

    reason: Optional[str] = None

    result: Optional[AsyncJobResponseResult] = None
    """Response from classify job - returned when polling /job/{job_id}"""


EnhancedAsyncJobResponseResult: TypeAlias = Union[
    ParseResponse, ExtractResponse, SplitResponse, EditResponse, PipelineResponse, V3Extract, ClassifyResponse, None
]


class EnhancedAsyncJobResponse(BaseModel):
    status: Literal["Pending", "Completed", "Failed", "Idle"]

    bucket: Optional[object] = None

    created_at: Optional[datetime] = None

    duration: Optional[float] = None

    num_pages: Optional[int] = None

    progress: Optional[float] = None

    raw_config: Optional[str] = None

    reason: Optional[str] = None

    result: Optional[EnhancedAsyncJobResponseResult] = None
    """Response from classify job - returned when polling /job/{job_id}"""

    source: Optional[object] = None

    total_pages: Optional[int] = None

    type: Optional[Literal["Parse", "Extract", "Split", "Edit", "Pipeline", "Classify"]] = None


JobGetResponse: TypeAlias = Union[AsyncJobResponse, EnhancedAsyncJobResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .bounding_box_param import BoundingBoxParam

__all__ = ["EditWidgetParam"]


class EditWidgetParam(TypedDict, total=False):
    bbox: Required[BoundingBoxParam]
    """Bounding box coordinates of the widget"""

    description: Required[str]
    """Description of the widget extracted from the document"""

    type: Required[Literal["text", "checkbox", "radio", "dropdown", "barcode"]]
    """Type of the form widget"""

    fill: bool
    """If True (default), the system will attempt to fill this widget.

    If False, the widget will be created but intentionally left unfilled.
    """

    font_size: Optional[float]
    """Font size in points for this specific field.

    Takes priority over the global font_size in EditOptions. If not set, falls back
    to the global font_size, then to auto-calculated sizing.
    """

    value: Optional[str]
    """
    If provided, this value will be used directly instead of attempting to
    intelligently determine the field value.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ArrayExtractConfig"]


class ArrayExtractConfig(BaseModel):
    enabled: Optional[bool] = None
    """
    Array extraction allows you to extract long lists of information from lengthy
    documents. It makes parallel calls on overlapping sections of the document.
    """

    mode: Optional[Literal["auto", "legacy", "streaming", "no_overlap"]] = None
    """The array extraction version to use."""

    pages_per_segment: Optional[int] = None
    """Length of each segment, in pages, for parallel calls with array extraction."""

    streaming_extract_item_density: Optional[int] = None
    """Number of items to extract in each stream call.

    Lower numbers will increase quality but be much slower. 50 works well for most
    documents with tables.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["EditOptionsParam"]


class EditOptionsParam(TypedDict, total=False):
    color: str
    """The color to use for edits, in hex format."""

    enable_overflow_pages: bool
    """If True, creates overflow pages for text that doesn't fit in form fields.

    Defaults to False.
    """

    flatten: bool
    """If True, flattens form fields after filling, converting them to static content.

    Defaults to False.
    """

    font_size: Optional[float]
    """The font size (in points) to use for filled text fields.

    If not specified, font size is automatically calculated based on field
    dimensions.
    """

    llm_provider_preference: Optional[Literal["openai", "anthropic", "google"]]
    """The LLM provider to use for edit processing.

    If not specified, defaults to 'google'
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TextAgentic"]


class TextAgentic(TypedDict, total=False):
    scope: Required[Literal["text"]]

    prompt: Optional[str]
    """Custom instructions for agentic text.

    Note: This only applies to form regions (key-value).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .job import (
    JobResource,
    AsyncJobResource,
    JobResourceWithRawResponse,
    AsyncJobResourceWithRawResponse,
    JobResourceWithStreamingResponse,
    AsyncJobResourceWithStreamingResponse,
)
from .edit import (
    EditResource,
    AsyncEditResource,
    EditResourceWithRawResponse,
    AsyncEditResourceWithRawResponse,
    EditResourceWithStreamingResponse,
    AsyncEditResourceWithStreamingResponse,
)
from .parse import (
    ParseResource,
    AsyncParseResource,
    ParseResourceWithRawResponse,
    AsyncParseResourceWithRawResponse,
    ParseResourceWithStreamingResponse,
    AsyncParseResourceWithStreamingResponse,
)
from .split import (
    SplitResource,
    AsyncSplitResource,
    SplitResourceWithRawResponse,
    AsyncSplitResourceWithRawResponse,
    SplitResourceWithStreamingResponse,
    AsyncSplitResourceWithStreamingResponse,
)
from .extract import (
    ExtractResource,
    AsyncExtractResource,
    ExtractResourceWithRawResponse,
    AsyncExtractResourceWithRawResponse,
    ExtractResourceWithStreamingResponse,
    AsyncExtractResourceWithStreamingResponse,
)
from .webhook import (
    WebhookResource,
    AsyncWebhookResource,
    WebhookResourceWithRawResponse,
    AsyncWebhookResourceWithRawResponse,
    WebhookResourceWithStreamingResponse,
    AsyncWebhookResourceWithStreamingResponse,
)
from .classify import (
    ClassifyResource,
    AsyncClassifyResource,
    ClassifyResourceWithRawResponse,
    AsyncClassifyResourceWithRawResponse,
    ClassifyResourceWithStreamingResponse,
    AsyncClassifyResourceWithStreamingResponse,
)
from .pipeline import (
    PipelineResource,
    AsyncPipelineResource,
    PipelineResourceWithRawResponse,
    AsyncPipelineResourceWithRawResponse,
    PipelineResourceWithStreamingResponse,
    AsyncPipelineResourceWithStreamingResponse,
)

__all__ = [
    "ParseResource",
    "AsyncParseResource",
    "ParseResourceWithRawResponse",
    "AsyncParseResourceWithRawResponse",
    "ParseResourceWithStreamingResponse",
    "AsyncParseResourceWithStreamingResponse",
    "ExtractResource",
    "AsyncExtractResource",
    "ExtractResourceWithRawResponse",
    "AsyncExtractResourceWithRawResponse",
    "ExtractResourceWithStreamingResponse",
    "AsyncExtractResourceWithStreamingResponse",
    "SplitResource",
    "AsyncSplitResource",
    "SplitResourceWithRawResponse",
    "AsyncSplitResourceWithRawResponse",
    "SplitResourceWithStreamingResponse",
    "AsyncSplitResourceWithStreamingResponse",
    "EditResource",
    "AsyncEditResource",
    "EditResourceWithRawResponse",
    "AsyncEditResourceWithRawResponse",
    "EditResourceWithStreamingResponse",
    "AsyncEditResourceWithStreamingResponse",
    "PipelineResource",
    "AsyncPipelineResource",
    "PipelineResourceWithRawResponse",
    "AsyncPipelineResourceWithRawResponse",
    "PipelineResourceWithStreamingResponse",
    "AsyncPipelineResourceWithStreamingResponse",
    "ClassifyResource",
    "AsyncClassifyResource",
    "ClassifyResourceWithRawResponse",
    "AsyncClassifyResourceWithRawResponse",
    "ClassifyResourceWithStreamingResponse",
    "AsyncClassifyResourceWithStreamingResponse",
    "WebhookResource",
    "AsyncWebhookResource",
    "WebhookResourceWithRawResponse",
    "AsyncWebhookResourceWithRawResponse",
    "WebhookResourceWithStreamingResponse",
    "AsyncWebhookResourceWithStreamingResponse",
    "JobResource",
    "AsyncJobResource",
    "JobResourceWithRawResponse",
    "AsyncJobResourceWithRawResponse",
    "JobResourceWithStreamingResponse",
    "AsyncJobResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr

__all__ = ["AsyncConfigV3Param", "Webhook", "WebhookSvixWebhookConfig", "WebhookDirectWebhookConfig"]


class WebhookSvixWebhookConfig(TypedDict, total=False):
    channels: SequenceNotStr[str]
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    mode: Literal["svix"]


class WebhookDirectWebhookConfig(TypedDict, total=False):
    url: Required[str]

    mode: Literal["direct"]


Webhook: TypeAlias = Union[WebhookSvixWebhookConfig, WebhookDirectWebhookConfig]


class AsyncConfigV3Param(TypedDict, total=False):
    metadata: object
    """JSON metadata included in webhook request body. Defaults to None."""

    priority: bool
    """
    If True, attempts to process the job with priority if the user has priority
    processing budget available; by default, sync jobs are prioritized above async
    jobs.
    """

    webhook: Optional[Webhook]
    """The webhook configuration for the asynchronous processing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ExtractSettingsParam", "Citations"]


class Citations(TypedDict, total=False):
    """The citations to use for the extraction."""

    enabled: bool
    """If True, include citations in the extraction."""

    numerical_confidence: bool
    """If True, enable numeric citation confidence scores. Defaults to True."""


class ExtractSettingsParam(TypedDict, total=False):
    array_extract: bool
    """If True, use array extraction."""

    citations: Citations
    """The citations to use for the extraction."""

    deep_extract: bool
    """
    If True, use Deep Extract, an agentic extraction mode that iteratively refines
    its output to achieve near-perfect accuracy. Best for complex documents where
    accuracy is critical.
    """

    include_images: bool
    """If True, include images in the extraction."""

    optimize_for_latency: bool
    """
    If True, jobs will be processed with a higher throughput and priority at a
    higher cost. Defaults to False.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["FigureSummaryConfig"]


class FigureSummaryConfig(BaseModel):
    advanced_chart_agent: Optional[bool] = None
    """If True, use the advanced chart agent. Defaults to False."""

    enabled: Optional[bool] = None
    """If figure summarization should be performed."""

    override: Optional[bool] = None
    """If the figure summary prompt should override our default prompt."""

    prompt: Optional[str] = None
    """Add information to the prompt for figure summarization.

    Note any visual cues that should be incorporated. Example: 'When provided a
    diagram, extract all of the figure content verbatim.'
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["InstructionsParam"]


class InstructionsParam(TypedDict, total=False):
    schema: object
    """The JSON schema to use for the extraction."""

    system_prompt: str
    """The system prompt to use for the extraction."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import TypedDict

from .._types import FileTypes

__all__ = ["ClientUploadParams"]


class ClientUploadParams(TypedDict, total=False):
    extension: Optional[str]

    file: Union[FileTypes, str, None]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from . import page_range
from ..._models import BaseModel
from .large_table_chunking_config import LargeTableChunkingConfig

__all__ = ["AdvancedProcessingOptions", "PageRange"]

PageRange: TypeAlias = Union[page_range.PageRange, List[page_range.PageRange], List[int], List[str]]


class AdvancedProcessingOptions(BaseModel):
    add_page_markers: Optional[bool] = None
    """If True, add page markers to the output (e.g.

    [[PAGE 1 BEGINS HERE]] and [[PAGE 1 ENDS HERE]] added as blocks to the content).
    Defaults to False.
    """

    continue_hierarchy: Optional[bool] = None
    """
    A flag to indicate if the hierarchy of the document should be continued from
    chunk to chunk.
    """

    document_password: Optional[str] = None
    """Password to decrypt password-protected documents."""

    enable_change_tracking: Optional[bool] = None
    """
    Enables model-based detection of underlines and strikethroughs, adding <u>/<s>
    tags to OCR text. Works with any extraction mode. Defaults to False.
    """

    enable_highlight_detection: Optional[bool] = None
    """If True, enable highlight detection.

    Highlighted text will be surrounded by <mark> tags in the output. Defaults to
    False.
    """

    exclude_hidden_rows_cols: Optional[bool] = None
    """Skip hidden rows and cols in Excel files. Defaults to False."""

    exclude_hidden_sheets: Optional[bool] = None
    """Skip hidden sheets in Excel files. Defaults to False."""

    filter_line_numbers: Optional[bool] = None
    """If True, filter out line numbers from the output. Defaults to False."""

    force_file_extension: Optional[str] = None
    """Force the URL to be downloaded as a specific file extension (e.g. .png)."""

    ignore_watermarks: Optional[bool] = None
    """If True, ignore and remove watermarks from OCR output. Defaults to False."""

    include_color_information: Optional[bool] = None
    """
    If True, preserve Excel cell colours in the extracted spreadsheet text using
    LaTeX colour commands.
    """

    include_dropdown_information: Optional[bool] = None
    """
    If True, include dropdown options and the selected value when rendering
    spreadsheet cells.
    """

    include_formula_information: Optional[bool] = None
    """
    If True, preserve formula information in spreadsheet cells by wrapping text with
    LaTeX formula commands during parsing.
    """

    keep_line_breaks: Optional[bool] = None
    """If line breaks should be preserved in the text."""

    large_table_chunking: Optional[LargeTableChunkingConfig] = None
    """
    The configuration options for large table chunking (currently only supported on
    spreadsheet and CSV files).
    """

    merge_tables: Optional[bool] = None
    """
    A flag to indicate if consecutive tables with the same number of columns should
    be merged across breaks and spaces.
    """

    ocr_system: Optional[Literal["highres", "multilingual", "combined", "reducto", "legacy", "reducto-v2"]] = None
    """The OCR system to use.

    Highres is recommended for documents with English characters. Legacy uses an
    alternative OCR backend.
    """

    page_range: Optional[PageRange] = None
    """The page range to process (1-indexed).

    By default, the entire document is processed. For spreadsheets, you can also
    provide a list of sheet names.
    """

    persist_results: Optional[bool] = None
    """If True, persist the results indefinitely. Defaults to False."""

    read_comments: Optional[bool] = None
    """If True, pull in PDF comments from the document. Defaults to False."""

    remove_text_formatting: Optional[bool] = None
    """If True, remove text formatting from the output (e.g.

    hyphens for list items). Defaults to False.
    """

    return_ocr_data: Optional[bool] = None
    """If True, return OCR data in the result. Defaults to False."""

    spreadsheet_table_clustering: Optional[Literal["default", "disabled", "intelligent"]] = None
    """
    In a spreadsheet with different tables inside, we enable splitting up the tables
    by default. Intelligent mode applies more powerful models for superior accuracy,
    at 5× the default per-cell rate. Disabling will register as one large table.
    """

    table_output_format: Optional[Literal["html", "json", "md", "jsonbbox", "dynamic", "ai_json", "csv"]] = None
    """The mode to use for table output.

    Dynamic returns md for simpler tables and html for more complex tables.
    """



from __future__ import annotations

import re
import json
from typing import Any, Union, Literal, Optional, cast
from pathlib import Path
from dataclasses import field, dataclass

import numpy as np
import numpy.typing as npt
from pydantic import BaseModel

from reducto.types import BoundingBox
from reducto.lib.helpers import ParseResponse, FullParseResponse, handle_url_response


def compute_iou_matrix(
    contained: npt.NDArray[Any],
    container: npt.NDArray[Any],
    mode: Literal["iou", "max", "containment", "edge_containment"] = "iou",
) -> npt.NDArray[np.float64]:
    """Calculate IoU or edge containment for all pairs of boxes between two lists.

    Args:
        contained (npt.NDArray[np.int32 | np.float32]): First list of boxes (e.g., proposals)
        container (npt.NDArray[np.int32 | np.float32]): Second list of boxes (e.g., ground truth)
        mode (str): Calculation mode ("iou", "max", "containment", or "edge_containment")

    Returns:
        npt.NDArray[np.float64]: Matrix of calculated values with shape [len(contained), len(container)]
    """
    x1 = np.maximum(contained[:, None, 0], container[:, 0])
    y1 = np.maximum(contained[:, None, 1], container[:, 1])
    x2 = np.minimum(contained[:, None, 2], container[:, 2])
    y2 = np.minimum(contained[:, None, 3], container[:, 3])

    overlap_w = np.maximum(0, x2 - x1)
    overlap_h = np.maximum(0, y2 - y1)
    intersection = overlap_w * overlap_h

    area1 = (contained[:, 2] - contained[:, 0]) * (contained[:, 3] - contained[:, 1])
    area2 = (container[:, 2] - container[:, 0]) * (container[:, 3] - container[:, 1])

    if mode == "max":
        denominator = np.minimum(area1[:, None], area2)
        result = np.divide(
            intersection,
            denominator,
            out=np.zeros_like(intersection, dtype=np.float64),
            where=denominator != 0,
        )
    elif mode == "containment":
        denominator = area1[:, None]
        result = np.divide(
            intersection,
            denominator,
            out=np.zeros_like(intersection, dtype=np.float64),
            where=denominator != 0,
        )
    elif mode == "edge_containment":
        width1 = contained[:, 2] - contained[:, 0]
        height1 = contained[:, 3] - contained[:, 1]

        percent_overlap_w = np.divide(
            overlap_w,
            width1[:, None],
            out=np.zeros_like(overlap_w, dtype=np.float64),
            where=width1[:, None] != 0,
        )
        percent_overlap_h = np.divide(
            overlap_h,
            height1[:, None],
            out=np.zeros_like(overlap_h, dtype=np.float64),
            where=height1[:, None] != 0,
        )

        result = np.minimum(percent_overlap_w, percent_overlap_h)
    else:
        # Default is IoU
        denominator = area1[:, None] + area2 - intersection
        result = np.divide(
            intersection,
            denominator,
            out=np.zeros_like(intersection, dtype=np.float64),
            where=denominator != 0,
        )
    return result


def normalize_string(text: str) -> str:
    """Normalize punctuation in a string."""
    normalized = re.sub(r"[^a-zA-Z0-9]", " ", text)
    normalized = " ".join(normalized.split())
    return normalized.lower()


class Citation(BaseModel):
    """A citation representing one occurrence of the target phrase."""

    page: int
    bboxes: list[BoundingBox]


@dataclass
class _PageDataBuilder:
    """Internal helper for building page data."""

    blocks: list[list[float]] = field(default_factory=lambda: [])
    lines: list[list[float]] = field(default_factory=lambda: [])
    words: list[list[float]] = field(default_factory=lambda: [])
    block_text: list[str] = field(default_factory=lambda: [])
    line_text: list[str] = field(default_factory=lambda: [])
    word_text: list[str] = field(default_factory=lambda: [])


@dataclass
class PageData:
    blocks: npt.NDArray[np.float64]  # n, xyxy (transform from xywh)
    lines: npt.NDArray[np.float64]  # n, xyxy (transform from xywh)
    words: npt.NDArray[np.float64]  # n, xyxy (transform from xywh)
    block_text: list[str]
    line_text: list[str]
    word_text: list[str]
    word_to_line: dict[int, int]
    line_to_block: dict[int, int]


class CitationFinder:
    def __init__(self, result: Union[Path, ParseResponse]) -> None:
        data: FullParseResponse
        if isinstance(result, Path):
            with open(result, "r") as f:
                parsed = ParseResponse.model_validate(json.load(f))
                data = handle_url_response(parsed)
        else:
            data = handle_url_response(result)

        self.data = data

        assert self.data.result.ocr is not None, "Enable return_ocr_data to continue."

        self.page_numbers, self.page_data = self._transform_ocr_to_page_data()

    def _transform_ocr_to_page_data(self) -> tuple[list[int], list[PageData]]:
        """Transform OCR data into PageData structure for each page for efficient computation."""

        pages_data: dict[int, _PageDataBuilder] = {}

        # Initialize page builders
        for chunk in self.data.result.chunks:
            for block in chunk.blocks:
                page_num = block.bbox.page
                if page_num not in pages_data:
                    pages_data[page_num] = _PageDataBuilder()

        # Collect block data
        for chunk in self.data.result.chunks:
            for block in chunk.blocks:
                page_num = block.bbox.page
                builder = pages_data[page_num]

                bbox = block.bbox
                xyxy_bbox = [
                    bbox.left,
                    bbox.top,
                    bbox.left + bbox.width,
                    bbox.top + bbox.height,
                ]

                builder.blocks.append(xyxy_bbox)
                builder.block_text.append(normalize_string(block.content))

        # Collect line and word data
        if self.data.result.ocr is not None:
            for line in self.data.result.ocr.lines:
                page_num = line.bbox.page
                builder = pages_data[page_num]

                bbox = line.bbox
                xyxy_bbox = [
                    bbox.left,
                    bbox.top,
                    bbox.left + bbox.width,
                    bbox.top + bbox.height,
                ]

                builder.lines.append(xyxy_bbox)
                builder.line_text.append(normalize_string(line.text))

            for word in self.data.result.ocr.words:
                page_num = word.bbox.page
                builder = pages_data[page_num]

                bbox = word.bbox
                xyxy_bbox = [
                    bbox.left,
                    bbox.top,
                    bbox.left + bbox.width,
                    bbox.top + bbox.height,
                ]

                builder.words.append(xyxy_bbox)
                builder.word_text.append(normalize_string(word.text))

        # Build PageData objects
        page_numbers = sorted(pages_data.keys())

        page_data_list: list[PageData] = []
        for page_num in page_numbers:
            builder = pages_data[page_num]
            blocks_arr = cast(npt.NDArray[np.float64], np.array(builder.blocks, dtype=np.float64).reshape(-1, 4))
            lines_arr = cast(npt.NDArray[np.float64], np.array(builder.lines, dtype=np.float64).reshape(-1, 4))
            words_arr = cast(npt.NDArray[np.float64], np.array(builder.words, dtype=np.float64).reshape(-1, 4))
            page_data = PageData(
                blocks=blocks_arr,
                lines=lines_arr,
                words=words_arr,
                block_text=builder.block_text,
                line_text=builder.line_text,
                word_text=builder.word_text,
                word_to_line={},
                line_to_block={},
            )
            page_data_list.append(page_data)

        for page_data in page_data_list:
            word_to_line: dict[int, int] = {}
            if page_data.words.shape[0] > 0 and page_data.lines.shape[0] > 0:
                word_line_matrix = compute_iou_matrix(page_data.words, page_data.lines, mode="containment")
                if word_line_matrix.size > 0:
                    word_to_line_indices = np.argmax(word_line_matrix, axis=1)
                    word_to_line = {
                        i: int(word_to_line_indices[i]) for i in range(len(word_to_line_indices))
                    }

            line_to_block: dict[int, int] = {}
            if page_data.lines.shape[0] > 0 and page_data.blocks.shape[0] > 0:
                line_block_matrix = compute_iou_matrix(page_data.lines, page_data.blocks, mode="containment")
                if line_block_matrix.size > 0:
                    line_to_block_indices = np.argmax(line_block_matrix, axis=1)
                    line_to_block = {
                        i: int(line_to_block_indices[i]) for i in range(len(line_to_block_indices))
                    }

            page_data.word_to_line = word_to_line
            page_data.line_to_block = line_to_block

        return page_numbers, page_data_list

    def cite(self, target: str, bbox_filter: Optional[BoundingBox] = None) -> list[Citation]:
        target = normalize_string(target)
        target_words = target.split()

        citations: list[Citation] = []

        matching_blocks: list[tuple[int, int, int, str]] = []
        for page_idx, page_data in enumerate(self.page_data):
            page_num = self.page_numbers[page_idx]
            for block_idx, block_text in enumerate(page_data.block_text):
                if bbox_filter is not None:
                    if bbox_filter.page != page_num:
                        continue

                    block_bbox = page_data.blocks[block_idx]

                    x1 = max(bbox_filter.left, block_bbox[0])
                    y1 = max(bbox_filter.top, block_bbox[1])
                    x2 = min(bbox_filter.left + bbox_filter.width, block_bbox[2])
                    y2 = min(bbox_filter.top + bbox_filter.height, block_bbox[3])

                    if x2 <= x1 or y2 <= y1:
                        continue

                    intersection_area = (x2 - x1) * (y2 - y1)
                    filter_area = bbox_filter.width * bbox_filter.height

                    if intersection_area / filter_area < 0.5:
                        continue

                if target in block_text:
                    matching_blocks.append((page_idx, page_num, block_idx, block_text))

        for page_idx, page_num, block_idx, _ in matching_blocks:
            page_data = self.page_data[page_idx]

            lines_in_block: list[int] = []
            for line_idx, mapped_block_idx in page_data.line_to_block.items():
                if mapped_block_idx == block_idx:
                    lines_in_block.append(line_idx)

            lines_in_block.sort(key=lambda line_idx: float(page_data.lines[line_idx][1]))

            words_in_block: list[str] = []
            word_indices_in_block: list[int] = []
            for line_idx in lines_in_block:
                words_in_line: list[int] = []
                for word_idx, mapped_line_idx in page_data.word_to_line.items():
                    if mapped_line_idx == line_idx:
                        words_in_line.append(word_idx)

                words_in_line.sort(key=lambda word_idx: float(page_data.words[word_idx][0]))

                for word_idx in words_in_line:
                    words_in_block.append(page_data.word_text[word_idx])
                    word_indices_in_block.append(word_idx)

            if len(target_words) == 0:
                continue

            match_start_indices: list[int] = []
            for i in range(len(words_in_block) - len(target_words) + 1):
                if words_in_block[i : i + len(target_words)] == target_words:
                    match_start_indices.append(i)

            for match_start_idx in match_start_indices:
                matching_word_indices = word_indices_in_block[match_start_idx : match_start_idx + len(target_words)]

                line_to_word_indices: dict[int, list[int]] = {}
                for word_idx in matching_word_indices:
                    line_idx = page_data.word_to_line[word_idx]
                    if line_idx not in line_to_word_indices:
                        line_to_word_indices[line_idx] = []
                    line_to_word_indices[line_idx].append(word_idx)

                line_bboxes: list[BoundingBox] = []
                for line_idx in sorted(line_to_word_indices.keys()):
                    word_indices = line_to_word_indices[line_idx]

                    word_boxes = page_data.words[word_indices]

                    left = np.min(word_boxes[:, 0])
                    top = np.min(word_boxes[:, 1])
                    right = np.max(word_boxes[:, 2])
                    bottom = np.max(word_boxes[:, 3])

                    bbox = BoundingBox(
                        page=page_num,
                        left=float(left),
                        top=float(top),
                        width=float(right - left),
                        height=float(bottom - top),
                    )
                    line_bboxes.append(bbox)

                citation = Citation(page=page_num, bboxes=line_bboxes)
                citations.append(citation)

        return citations



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .edit_widget_param import EditWidgetParam
from .edit_options_param import EditOptionsParam
from .shared_params.upload import Upload

__all__ = ["EditRunJobParams", "DocumentURL", "Webhook"]


class EditRunJobParams(TypedDict, total=False):
    document_url: Required[DocumentURL]
    """The URL of the document to be processed. You can provide one of the following:

    1. A publicly available URL
    2. A presigned S3 URL
    3. A reducto:// prefixed URL obtained from the /upload endpoint after directly
       uploading a document
    """

    edit_instructions: Required[str]
    """The instructions for the edit."""

    edit_options: EditOptionsParam

    form_schema: Optional[Iterable[EditWidgetParam]]
    """Form schema for PDF forms.

    List of widgets with their types, descriptions, and bounding boxes. Only works
    for PDFs.
    """

    priority: bool
    """
    If True, attempts to process the job with priority if the user has priority
    processing budget available; by default, sync jobs are prioritized above async
    jobs.
    """

    webhook: Webhook


DocumentURL: TypeAlias = Union[str, Upload]


class Webhook(TypedDict, total=False):
    channels: SequenceNotStr[str]
    """
    A list of Svix channels the message will be delivered down, omit to send to all
    channels.
    """

    metadata: object
    """JSON metadata included in webhook request body"""

    mode: Literal["disabled", "svix", "direct"]
    """The mode to use for webhook delivery.

    Defaults to 'disabled'. We recommend using 'svix' for production environments.
    """

    url: str
    """The URL to send the webhook to (if using direct webhoook)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["TableSummaryConfig"]


class TableSummaryConfig(BaseModel):
    enabled: Optional[bool] = None
    """If table summarization should be performed."""

    prompt: Optional[str] = None
    """Add information to the prompt for table summarization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TextAgentic"]


class TextAgentic(BaseModel):
    scope: Literal["text"]

    prompt: Optional[str] = None
    """Custom instructions for agentic text.

    Note: This only applies to form regions (key-value).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel
from .text_agentic import TextAgentic
from .table_agentic import TableAgentic
from .figure_agentic import FigureAgentic

__all__ = ["Enhance", "Agentic"]

Agentic: TypeAlias = Union[TableAgentic, FigureAgentic, TextAgentic]


class Enhance(BaseModel):
    agentic: Optional[List[Agentic]] = None
    """
    Agentic uses vision language models to enhance the accuracy of the output of
    different types of extraction. This will incur a cost and latency increase.
    """

    intelligent_ordering: Optional[bool] = None
    """
    If True, use an advanced vision language model to improve reading order
    accuracy, with a small increase in latency. Defaults to False.
    """

    summarize_figures: Optional[bool] = None
    """If True, summarize figures using a small vision language model.

    Defaults to True.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import pipeline_run_params, pipeline_run_job_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.pipeline_response import PipelineResponse
from ..types.async_config_v3_param import AsyncConfigV3Param
from ..types.pipeline_settings_param import PipelineSettingsParam
from ..types.pipeline_run_job_response import PipelineRunJobResponse

__all__ = ["PipelineResource", "AsyncPipelineResource"]


class PipelineResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PipelineResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return PipelineResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PipelineResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return PipelineResourceWithStreamingResponse(self)

    def run(
        self,
        *,
        input: pipeline_run_params.Input,
        pipeline_id: str,
        settings: PipelineSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineResponse:
        """
        Pipeline

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          pipeline_id: The ID of the pipeline to use for the document.

          settings: Settings for pipeline execution that override pipeline defaults.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/pipeline",
            body=maybe_transform(
                {
                    "input": input,
                    "pipeline_id": pipeline_id,
                    "settings": settings,
                },
                pipeline_run_params.PipelineRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineResponse,
        )

    def run_job(
        self,
        *,
        input: pipeline_run_job_params.Input,
        pipeline_id: str,
        async_: AsyncConfigV3Param | Omit = omit,
        settings: PipelineSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineRunJobResponse:
        """
        Pipeline Async

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          pipeline_id: The ID of the pipeline to use for the document.

          async_: The configuration options for asynchronous processing (default synchronous).

          settings: Settings for pipeline execution that override pipeline defaults.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/pipeline_async",
            body=maybe_transform(
                {
                    "input": input,
                    "pipeline_id": pipeline_id,
                    "async_": async_,
                    "settings": settings,
                },
                pipeline_run_job_params.PipelineRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineRunJobResponse,
        )


class AsyncPipelineResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPipelineResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncPipelineResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPipelineResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncPipelineResourceWithStreamingResponse(self)

    async def run(
        self,
        *,
        input: pipeline_run_params.Input,
        pipeline_id: str,
        settings: PipelineSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineResponse:
        """
        Pipeline

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          pipeline_id: The ID of the pipeline to use for the document.

          settings: Settings for pipeline execution that override pipeline defaults.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/pipeline",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "pipeline_id": pipeline_id,
                    "settings": settings,
                },
                pipeline_run_params.PipelineRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineResponse,
        )

    async def run_job(
        self,
        *,
        input: pipeline_run_job_params.Input,
        pipeline_id: str,
        async_: AsyncConfigV3Param | Omit = omit,
        settings: PipelineSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineRunJobResponse:
        """
        Pipeline Async

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          pipeline_id: The ID of the pipeline to use for the document.

          async_: The configuration options for asynchronous processing (default synchronous).

          settings: Settings for pipeline execution that override pipeline defaults.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/pipeline_async",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "pipeline_id": pipeline_id,
                    "async_": async_,
                    "settings": settings,
                },
                pipeline_run_job_params.PipelineRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PipelineRunJobResponse,
        )


class PipelineResourceWithRawResponse:
    def __init__(self, pipeline: PipelineResource) -> None:
        self._pipeline = pipeline

        self.run = to_raw_response_wrapper(
            pipeline.run,
        )
        self.run_job = to_raw_response_wrapper(
            pipeline.run_job,
        )


class AsyncPipelineResourceWithRawResponse:
    def __init__(self, pipeline: AsyncPipelineResource) -> None:
        self._pipeline = pipeline

        self.run = async_to_raw_response_wrapper(
            pipeline.run,
        )
        self.run_job = async_to_raw_response_wrapper(
            pipeline.run_job,
        )


class PipelineResourceWithStreamingResponse:
    def __init__(self, pipeline: PipelineResource) -> None:
        self._pipeline = pipeline

        self.run = to_streamed_response_wrapper(
            pipeline.run,
        )
        self.run_job = to_streamed_response_wrapper(
            pipeline.run_job,
        )


class AsyncPipelineResourceWithStreamingResponse:
    def __init__(self, pipeline: AsyncPipelineResource) -> None:
        self._pipeline = pipeline

        self.run = async_to_streamed_response_wrapper(
            pipeline.run,
        )
        self.run_job = async_to_streamed_response_wrapper(
            pipeline.run_job,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from .._models import BaseModel
from .v3_extract import V3Extract
from .parse_usage import ParseUsage
from .edit_response import EditResponse
from .parse_response import ParseResponse
from .split_response import SplitResponse
from .extract_response import ExtractResponse

__all__ = [
    "PipelineResponse",
    "Result",
    "ResultExtract",
    "ResultExtractExtractVariant0",
    "ResultExtractExtractVariant0Result",
    "ResultParse",
]

ResultExtractExtractVariant0Result: TypeAlias = Union[ExtractResponse, V3Extract]


class ResultExtractExtractVariant0(BaseModel):
    """This is the response format for Extract -> Split Pipelines"""

    page_range: List[int]

    result: ResultExtractExtractVariant0Result

    split_name: str

    partition: Optional[str] = None


ResultExtract: TypeAlias = Union[List[ResultExtractExtractVariant0], ExtractResponse, V3Extract, None]

ResultParse: TypeAlias = Union[ParseResponse, List[ParseResponse], None]


class Result(BaseModel):
    extract: Optional[ResultExtract] = None

    parse: Optional[ResultParse] = None

    split: Optional[SplitResponse] = None

    edit: Optional[EditResponse] = None


class PipelineResponse(BaseModel):
    job_id: str

    result: Result

    usage: ParseUsage



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from . import page_range
from ..._models import BaseModel

__all__ = ["Settings", "PageRange"]

PageRange: TypeAlias = Union[page_range.PageRange, List[page_range.PageRange], List[int], List[str], None]


class Settings(BaseModel):
    document_password: Optional[str] = None
    """Password to decrypt password-protected documents."""

    embed_pdf_metadata: Optional[bool] = None
    """If True, embed OCR metadata into the returned PDF. Defaults to False."""

    extraction_mode: Optional[Literal["ocr", "hybrid"]] = None
    """The mode to use for text extraction from PDFs.

    OCR mode uses optical character recognition only. Hybrid mode combines OCR with
    embedded PDF text for best accuracy (default).
    """

    force_file_extension: Optional[str] = None
    """Force the URL to be downloaded as a specific file extension (e.g. `.png`)."""

    force_url_result: Optional[bool] = None
    """Force the result to be returned in URL form."""

    ocr_system: Optional[Literal["standard", "legacy"]] = None
    """Standard is our best multilingual OCR system.

    Legacy only supports germanic languages and is available for backwards
    compatibility.
    """

    page_range: Optional[PageRange] = None
    """The page range to process (1-indexed).

    By default, the entire document is processed. For spreadsheets, you can also
    provide a list of sheet names.
    """

    persist_results: Optional[bool] = None
    """If True, persist the results indefinitely. Defaults to False."""

    return_images: Optional[List[Literal["figure", "table"]]] = None
    """Whether to return images for the specified block types.

    By default, no images are returned.
    """

    return_ocr_data: Optional[bool] = None
    """If True, return OCR data in the result. Defaults to False."""

    timeout: Optional[float] = None
    """The timeout for the job in seconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .split_large_tables import SplitLargeTables

__all__ = ["Spreadsheet"]


class Spreadsheet(TypedDict, total=False):
    clustering: Literal["accurate", "fast", "disabled"]
    """
    In a spreadsheet with different tables inside, we enable splitting up the tables
    by default. Accurate mode applies more powerful models for superior accuracy, at
    5× the default per-cell rate. Disabling will register as one large table.
    """

    exclude: List[Literal["hidden_sheets", "hidden_rows", "hidden_cols", "styling", "spreadsheet_images"]]
    """Whether to exclude hidden sheets, rows, or columns in the output."""

    include: List[Literal["cell_colors", "formula", "dropdowns"]]
    """Whether to include cell color, formula, and dropdown information in the output."""

    split_large_tables: SplitLargeTables



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "ClassifyResponse",
    "Result",
    "ResponseConfidence",
    "ResponseConfidenceCategory",
    "ResponseConfidenceCategoryCriteriaConfidence",
]


class Result(BaseModel):
    category: str


class ResponseConfidenceCategoryCriteriaConfidence(BaseModel):
    """Confidence result for a single criterion."""

    confidence: Literal["high", "low"]

    criterion: str


class ResponseConfidenceCategory(BaseModel):
    """Confidence result for a category."""

    category: str

    confidence: float

    criteria_confidence: List[ResponseConfidenceCategoryCriteriaConfidence]


class ResponseConfidence(BaseModel):
    """Overall confidence breakdown for classification response."""

    categories: List[ResponseConfidenceCategory]


class ClassifyResponse(BaseModel):
    """Response from classify job - returned when polling /job/{job_id}"""

    job_id: str

    result: Result

    duration: Optional[float] = None
    """The duration of the classify request in seconds."""

    response_confidence: Optional[ResponseConfidence] = None
    """Overall confidence breakdown for classification response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .shared import Upload as Upload
from .v3_extract import V3Extract as V3Extract
from .edit_widget import EditWidget as EditWidget
from .parse_usage import ParseUsage as ParseUsage
from .bounding_box import BoundingBox as BoundingBox
from .edit_response import EditResponse as EditResponse
from .enhance_param import EnhanceParam as EnhanceParam
from .extract_usage import ExtractUsage as ExtractUsage
from .parse_response import ParseResponse as ParseResponse
from .settings_param import SettingsParam as SettingsParam
from .split_response import SplitResponse as SplitResponse
from .edit_run_params import EditRunParams as EditRunParams
from .retrieval_param import RetrievalParam as RetrievalParam
from .extract_response import ExtractResponse as ExtractResponse
from .formatting_param import FormattingParam as FormattingParam
from .job_get_response import JobGetResponse as JobGetResponse
from .page_range_param import PageRangeParam as PageRangeParam
from .parse_run_params import ParseRunParams as ParseRunParams
from .split_run_params import SplitRunParams as SplitRunParams
from .classify_response import ClassifyResponse as ClassifyResponse
from .edit_widget_param import EditWidgetParam as EditWidgetParam
from .pipeline_response import PipelineResponse as PipelineResponse
from .spreadsheet_param import SpreadsheetParam as SpreadsheetParam
from .bounding_box_param import BoundingBoxParam as BoundingBoxParam
from .edit_options_param import EditOptionsParam as EditOptionsParam
from .extract_run_params import ExtractRunParams as ExtractRunParams
from .instructions_param import InstructionsParam as InstructionsParam
from .job_get_all_params import JobGetAllParams as JobGetAllParams
from .parse_run_response import ParseRunResponse as ParseRunResponse
from .classify_run_params import ClassifyRunParams as ClassifyRunParams
from .edit_run_job_params import EditRunJobParams as EditRunJobParams
from .parse_options_param import ParseOptionsParam as ParseOptionsParam
from .pipeline_run_params import PipelineRunParams as PipelineRunParams
from .api_version_response import APIVersionResponse as APIVersionResponse
from .async_parse_response import AsyncParseResponse as AsyncParseResponse
from .client_upload_params import ClientUploadParams as ClientUploadParams
from .extract_run_response import ExtractRunResponse as ExtractRunResponse
from .job_get_all_response import JobGetAllResponse as JobGetAllResponse
from .parse_run_job_params import ParseRunJobParams as ParseRunJobParams
from .split_category_param import SplitCategoryParam as SplitCategoryParam
from .split_run_job_params import SplitRunJobParams as SplitRunJobParams
from .webhook_run_response import WebhookRunResponse as WebhookRunResponse
from .async_config_v3_param import AsyncConfigV3Param as AsyncConfigV3Param
from .edit_run_job_response import EditRunJobResponse as EditRunJobResponse
from .async_extract_response import AsyncExtractResponse as AsyncExtractResponse
from .extract_run_job_params import ExtractRunJobParams as ExtractRunJobParams
from .extract_settings_param import ExtractSettingsParam as ExtractSettingsParam
from .split_run_job_response import SplitRunJobResponse as SplitRunJobResponse
from .pipeline_run_job_params import PipelineRunJobParams as PipelineRunJobParams
from .pipeline_settings_param import PipelineSettingsParam as PipelineSettingsParam
from .async_parse_config_param import AsyncParseConfigParam as AsyncParseConfigParam
from .deep_split_page_evidence import DeepSplitPageEvidence as DeepSplitPageEvidence
from .pipeline_run_job_response import PipelineRunJobResponse as PipelineRunJobResponse
from .split_table_options_param import SplitTableOptionsParam as SplitTableOptionsParam
from .async_extract_config_param import AsyncExtractConfigParam as AsyncExtractConfigParam



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ExtractUsage"]


class ExtractUsage(BaseModel):
    num_fields: int

    num_pages: int

    credits: Optional[float] = None

    extract_mode: Optional[Literal["super_agent", "extract", "spreadsheet_agent"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .instructions_param import InstructionsParam
from .parse_options_param import ParseOptionsParam
from .shared_params.upload import Upload
from .async_config_v3_param import AsyncConfigV3Param
from .extract_settings_param import ExtractSettingsParam

__all__ = ["ExtractRunJobParams", "Input"]


class ExtractRunJobParams(TypedDict, total=False):
    input: Required[Input]
    """For parse/split/extract pipelines, the URL of the document to be processed.

    You can provide one of the following: 1. A publicly available URL 2. A presigned
    S3 URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
    directly uploading a document 4. A jobid:// prefixed URL obtained from a
    previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
    API only)

                For edit pipelines, this should be a string containing the edit instructions
    """

    async_: Annotated[AsyncConfigV3Param, PropertyInfo(alias="async")]
    """The configuration options for asynchronous processing (default synchronous)."""

    instructions: InstructionsParam
    """The instructions to use for the extraction."""

    parsing: ParseOptionsParam
    """The configuration options for parsing the document.

    If you are passing in a jobid:// URL for the file, then this configuration will
    be ignored.
    """

    settings: ExtractSettingsParam
    """The settings to use for the extraction."""


Input: TypeAlias = Union[str, SequenceNotStr[str], Upload]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast
from typing_extensions import Literal, overload

import httpx

from ..types import (
    parse_run_params,
    parse_run_job_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.enhance_param import EnhanceParam
from ..types.settings_param import SettingsParam
from ..types.retrieval_param import RetrievalParam
from ..types.formatting_param import FormattingParam
from ..types.spreadsheet_param import SpreadsheetParam
from ..types.parse_run_response import ParseRunResponse
from ..types.async_parse_response import AsyncParseResponse
from ..types.async_config_v3_param import AsyncConfigV3Param

__all__ = ["ParseResource", "AsyncParseResource"]


class ParseResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ParseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ParseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ParseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return ParseResourceWithStreamingResponse(self)

    @overload
    def run(
        self,
        *,
        input: parse_run_params.SyncParseConfigInput,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        """
        Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def run(
        self,
        *,
        input: parse_run_params.AsyncParseConfigInput,
        async_: AsyncConfigV3Param | Omit = omit,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        """
        Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          queue_priority: Queue priority. 'batch' for non-urgent work that processes when spare GPU
              capacity is available.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input"])
    def run(
        self,
        *,
        input: parse_run_params.SyncParseConfigInput | parse_run_params.AsyncParseConfigInput,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        async_: AsyncConfigV3Param | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        return cast(
            ParseRunResponse,
            self._post(
                "/parse",
                body=maybe_transform(
                    {
                        "input": input,
                        "enhance": enhance,
                        "formatting": formatting,
                        "retrieval": retrieval,
                        "settings": settings,
                        "spreadsheet": spreadsheet,
                        "async_": async_,
                        "queue_priority": queue_priority,
                    },
                    parse_run_params.ParseRunParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ParseRunResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def run_job(
        self,
        *,
        input: parse_run_job_params.Input,
        async_: AsyncConfigV3Param | Omit = omit,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncParseResponse:
        """
        Async Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          queue_priority: Queue priority. 'batch' for non-urgent work that processes when spare GPU
              capacity is available.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/parse_async",
            body=maybe_transform(
                {
                    "input": input,
                    "async_": async_,
                    "enhance": enhance,
                    "formatting": formatting,
                    "queue_priority": queue_priority,
                    "retrieval": retrieval,
                    "settings": settings,
                    "spreadsheet": spreadsheet,
                },
                parse_run_job_params.ParseRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncParseResponse,
        )


class AsyncParseResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncParseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncParseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncParseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncParseResourceWithStreamingResponse(self)

    @overload
    async def run(
        self,
        *,
        input: parse_run_params.SyncParseConfigInput,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        """
        Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def run(
        self,
        *,
        input: parse_run_params.AsyncParseConfigInput,
        async_: AsyncConfigV3Param | Omit = omit,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        """
        Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          queue_priority: Queue priority. 'batch' for non-urgent work that processes when spare GPU
              capacity is available.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input"])
    async def run(
        self,
        *,
        input: parse_run_params.SyncParseConfigInput | parse_run_params.AsyncParseConfigInput,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        async_: AsyncConfigV3Param | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ParseRunResponse:
        return cast(
            ParseRunResponse,
            await self._post(
                "/parse",
                body=await async_maybe_transform(
                    {
                        "input": input,
                        "enhance": enhance,
                        "formatting": formatting,
                        "retrieval": retrieval,
                        "settings": settings,
                        "spreadsheet": spreadsheet,
                        "async_": async_,
                        "queue_priority": queue_priority,
                    },
                    parse_run_params.ParseRunParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(Any, ParseRunResponse),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def run_job(
        self,
        *,
        input: parse_run_job_params.Input,
        async_: AsyncConfigV3Param | Omit = omit,
        enhance: EnhanceParam | Omit = omit,
        formatting: FormattingParam | Omit = omit,
        queue_priority: Literal["auto", "batch"] | Omit = omit,
        retrieval: RetrievalParam | Omit = omit,
        settings: SettingsParam | Omit = omit,
        spreadsheet: SpreadsheetParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncParseResponse:
        """
        Async Parse

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          async_: The configuration options for asynchronous processing (default synchronous).

          queue_priority: Queue priority. 'batch' for non-urgent work that processes when spare GPU
              capacity is available.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/parse_async",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "async_": async_,
                    "enhance": enhance,
                    "formatting": formatting,
                    "queue_priority": queue_priority,
                    "retrieval": retrieval,
                    "settings": settings,
                    "spreadsheet": spreadsheet,
                },
                parse_run_job_params.ParseRunJobParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncParseResponse,
        )


class ParseResourceWithRawResponse:
    def __init__(self, parse: ParseResource) -> None:
        self._parse = parse

        self.run = to_raw_response_wrapper(
            parse.run,
        )
        self.run_job = to_raw_response_wrapper(
            parse.run_job,
        )


class AsyncParseResourceWithRawResponse:
    def __init__(self, parse: AsyncParseResource) -> None:
        self._parse = parse

        self.run = async_to_raw_response_wrapper(
            parse.run,
        )
        self.run_job = async_to_raw_response_wrapper(
            parse.run_job,
        )


class ParseResourceWithStreamingResponse:
    def __init__(self, parse: ParseResource) -> None:
        self._parse = parse

        self.run = to_streamed_response_wrapper(
            parse.run,
        )
        self.run_job = to_streamed_response_wrapper(
            parse.run_job,
        )


class AsyncParseResourceWithStreamingResponse:
    def __init__(self, parse: AsyncParseResource) -> None:
        self._parse = parse

        self.run = async_to_streamed_response_wrapper(
            parse.run,
        )
        self.run_job = async_to_streamed_response_wrapper(
            parse.run_job,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .chunking import Chunking

__all__ = ["Retrieval"]


class Retrieval(TypedDict, total=False):
    chunking: Chunking

    embedding_optimized: bool
    """If True, use embedding optimized mode. Defaults to False."""

    filter_blocks: List[
        Literal[
            "Header",
            "Footer",
            "Title",
            "Section Header",
            "Page Number",
            "List Item",
            "Figure",
            "Table",
            "Key Value",
            "Text",
            "Comment",
            "Signature",
        ]
    ]
    """A list of block types to filter out from 'content' and 'embed' fields.

    By default, no blocks are filtered.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reducto import Reducto, AsyncReducto
from tests.utils import assert_matches_type
from reducto.types import (
    PipelineResponse,
    PipelineRunJobResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPipeline:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run(self, client: Reducto) -> None:
        pipeline = client.pipeline.run(
            input="string",
            pipeline_id="pipeline_id",
        )
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params(self, client: Reducto) -> None:
        pipeline = client.pipeline.run(
            input="string",
            pipeline_id="pipeline_id",
            settings={"document_password": "document_password"},
        )
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run(self, client: Reducto) -> None:
        response = client.pipeline.with_raw_response.run(
            input="string",
            pipeline_id="pipeline_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run(self, client: Reducto) -> None:
        with client.pipeline.with_streaming_response.run(
            input="string",
            pipeline_id="pipeline_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(PipelineResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job(self, client: Reducto) -> None:
        pipeline = client.pipeline.run_job(
            input="string",
            pipeline_id="pipeline_id",
        )
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job_with_all_params(self, client: Reducto) -> None:
        pipeline = client.pipeline.run_job(
            input="string",
            pipeline_id="pipeline_id",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            settings={"document_password": "document_password"},
        )
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_job(self, client: Reducto) -> None:
        response = client.pipeline.with_raw_response.run_job(
            input="string",
            pipeline_id="pipeline_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = response.parse()
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_job(self, client: Reducto) -> None:
        with client.pipeline.with_streaming_response.run_job(
            input="string",
            pipeline_id="pipeline_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = response.parse()
            assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncPipeline:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run(self, async_client: AsyncReducto) -> None:
        pipeline = await async_client.pipeline.run(
            input="string",
            pipeline_id="pipeline_id",
        )
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params(self, async_client: AsyncReducto) -> None:
        pipeline = await async_client.pipeline.run(
            input="string",
            pipeline_id="pipeline_id",
            settings={"document_password": "document_password"},
        )
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncReducto) -> None:
        response = await async_client.pipeline.with_raw_response.run(
            input="string",
            pipeline_id="pipeline_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(PipelineResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run(self, async_client: AsyncReducto) -> None:
        async with async_client.pipeline.with_streaming_response.run(
            input="string",
            pipeline_id="pipeline_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(PipelineResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job(self, async_client: AsyncReducto) -> None:
        pipeline = await async_client.pipeline.run_job(
            input="string",
            pipeline_id="pipeline_id",
        )
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job_with_all_params(self, async_client: AsyncReducto) -> None:
        pipeline = await async_client.pipeline.run_job(
            input="string",
            pipeline_id="pipeline_id",
            async_={
                "metadata": {},
                "priority": True,
                "webhook": {
                    "channels": ["string"],
                    "mode": "svix",
                },
            },
            settings={"document_password": "document_password"},
        )
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_job(self, async_client: AsyncReducto) -> None:
        response = await async_client.pipeline.with_raw_response.run_job(
            input="string",
            pipeline_id="pipeline_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        pipeline = await response.parse()
        assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_job(self, async_client: AsyncReducto) -> None:
        async with async_client.pipeline.with_streaming_response.run_job(
            input="string",
            pipeline_id="pipeline_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            pipeline = await response.parse()
            assert_matches_type(PipelineRunJobResponse, pipeline, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["JobGetAllResponse", "Job"]


class Job(BaseModel):
    created_at: datetime

    duration: Optional[float] = None

    job_id: str

    num_pages: Optional[int] = None

    raw_config: str

    status: Literal["Pending", "Completed", "Failed", "Idle", "InProgress", "Completing", "Cancelled"]

    total_pages: Optional[int] = None

    type: Literal["Parse", "Extract", "Split", "Edit", "Pipeline", "Classify"]

    bucket: Optional[object] = None

    source: Optional[object] = None


class JobGetAllResponse(BaseModel):
    jobs: List[Job]
    """
    List of jobs with their job_id, status, type, raw_config, created_at, num_pages
    and duration
    """

    next_cursor: Optional[str] = None
    """Cursor to fetch the next page of results. If null, there are no more results."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TableAgentic"]


class TableAgentic(BaseModel):
    scope: Literal["table"]

    prompt: Optional[str] = None
    """Custom prompt for table agentic."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PageRange"]


class PageRange(BaseModel):
    end: Optional[int] = None
    """The page number to stop processing at (1-indexed)."""

    start: Optional[int] = None
    """The page number to start processing from (1-indexed)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["JobGetAllParams"]


class JobGetAllParams(TypedDict, total=False):
    cursor: Optional[str]
    """Cursor for pagination.

    Use the next_cursor from the previous response to fetch the next page.
    """

    exclude_configs: bool
    """Exclude raw_config from response to reduce size"""

    limit: int
    """Maximum number of jobs to return per page. Defaults to 100, max 500."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .enhance_param import EnhanceParam
from .settings_param import SettingsParam
from .retrieval_param import RetrievalParam
from .formatting_param import FormattingParam
from .spreadsheet_param import SpreadsheetParam
from .shared_params.upload import Upload
from .async_config_v3_param import AsyncConfigV3Param

__all__ = ["ParseRunParams", "SyncParseConfig", "SyncParseConfigInput", "AsyncParseConfig", "AsyncParseConfigInput"]


class SyncParseConfig(TypedDict, total=False):
    input: Required[SyncParseConfigInput]
    """For parse/split/extract pipelines, the URL of the document to be processed.

    You can provide one of the following: 1. A publicly available URL 2. A presigned
    S3 URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
    directly uploading a document 4. A jobid:// prefixed URL obtained from a
    previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
    API only)

                For edit pipelines, this should be a string containing the edit instructions
    """

    enhance: EnhanceParam

    formatting: FormattingParam

    retrieval: RetrievalParam

    settings: SettingsParam

    spreadsheet: SpreadsheetParam


SyncParseConfigInput: TypeAlias = Union[str, SequenceNotStr[str], Upload]


class AsyncParseConfig(TypedDict, total=False):
    input: Required[AsyncParseConfigInput]
    """For parse/split/extract pipelines, the URL of the document to be processed.

    You can provide one of the following: 1. A publicly available URL 2. A presigned
    S3 URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
    directly uploading a document 4. A jobid:// prefixed URL obtained from a
    previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
    API only)

                For edit pipelines, this should be a string containing the edit instructions
    """

    async_: Annotated[AsyncConfigV3Param, PropertyInfo(alias="async")]
    """The configuration options for asynchronous processing (default synchronous)."""

    enhance: EnhanceParam

    formatting: FormattingParam

    queue_priority: Literal["auto", "batch"]
    """Queue priority.

    'batch' for non-urgent work that processes when spare GPU capacity is available.
    """

    retrieval: RetrievalParam

    settings: SettingsParam

    spreadsheet: SpreadsheetParam


AsyncParseConfigInput: TypeAlias = Union[str, SequenceNotStr[str], Upload]

ParseRunParams: TypeAlias = Union[SyncParseConfig, AsyncParseConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .chunking import Chunking
from ..._models import BaseModel

__all__ = ["Retrieval"]


class Retrieval(BaseModel):
    chunking: Optional[Chunking] = None

    embedding_optimized: Optional[bool] = None
    """If True, use embedding optimized mode. Defaults to False."""

    filter_blocks: Optional[
        List[
            Literal[
                "Header",
                "Footer",
                "Title",
                "Section Header",
                "Page Number",
                "List Item",
                "Figure",
                "Table",
                "Key Value",
                "Text",
                "Comment",
                "Signature",
            ]
        ]
    ] = None
    """A list of block types to filter out from 'content' and 'embed' fields.

    By default, no blocks are filtered.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["EnhanceParam", "Agentic", "AgenticTableAgentic", "AgenticFigureAgentic", "AgenticTextAgentic"]


class AgenticTableAgentic(TypedDict, total=False):
    scope: Required[Literal["table"]]

    prompt: Optional[str]
    """Custom prompt for table agentic."""


class AgenticFigureAgentic(TypedDict, total=False):
    scope: Required[Literal["figure"]]

    advanced_chart_agent: bool
    """If True, use the advanced chart agent. Defaults to False."""

    prompt: Optional[str]
    """Custom prompt for figure agentic."""

    return_overlays: bool
    """If True, return overlays for the figure.

    This is so you can use the overlays to double check the quality of the
    extraction
    """


class AgenticTextAgentic(TypedDict, total=False):
    scope: Required[Literal["text"]]

    prompt: Optional[str]
    """Custom instructions for agentic text.

    Note: This only applies to form regions (key-value).
    """


Agentic: TypeAlias = Union[AgenticTableAgentic, AgenticFigureAgentic, AgenticTextAgentic]


class EnhanceParam(TypedDict, total=False):
    agentic: Iterable[Agentic]
    """
    Agentic uses vision language models to enhance the accuracy of the output of
    different types of extraction. This will incur a cost and latency increase.
    """

    intelligent_ordering: bool
    """
    If True, use an advanced vision language model to improve reading order
    accuracy, with a small increase in latency. Defaults to False.
    """

    summarize_figures: bool
    """If True, summarize figures using a small vision language model.

    Defaults to True.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["AdvancedCitationsConfig"]


class AdvancedCitationsConfig(BaseModel):
    numerical_confidence: Optional[bool] = None
    """If True, enable numeric citation confidence scores. Defaults to False."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .enhance import Enhance
from .settings import Settings
from .retrieval import Retrieval
from .formatting import Formatting
from .spreadsheet import Spreadsheet

__all__ = ["ParseOptions"]


class ParseOptions(TypedDict, total=False):
    enhance: Enhance

    formatting: Formatting

    retrieval: Retrieval

    settings: Settings

    spreadsheet: Spreadsheet



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["SplitTableOptionsParam"]


class SplitTableOptionsParam(TypedDict, total=False):
    table_cutoff: Literal["truncate", "preserve"]
    """
    If tables should be truncated to the first few rows or if all content should be
    preserved. truncate improves latency, preserve is recommended for cases where
    partition_key is being used and the partition_key may be included within the
    table. Defaults to truncate
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import TypeAlias

from .v3_extract import V3Extract
from .async_extract_response import AsyncExtractResponse

__all__ = ["ExtractRunResponse"]

ExtractRunResponse: TypeAlias = Union[V3Extract, AsyncExtractResponse]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional

import httpx

from ..types import classify_run_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.classify_response import ClassifyResponse

__all__ = ["ClassifyResource", "AsyncClassifyResource"]


class ClassifyResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClassifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ClassifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClassifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return ClassifyResourceWithStreamingResponse(self)

    def run(
        self,
        *,
        input: classify_run_params.Input,
        classification_schema: Iterable[classify_run_params.ClassificationSchema] | Omit = omit,
        document_metadata: Optional[str] | Omit = omit,
        page_range: Optional[classify_run_params.PageRange] | Omit = omit,
        persist_results: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClassifyResponse:
        """
        Classify

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          classification_schema: A list of classification categories and their matching criteria.

          document_metadata: Optional document-level metadata to include in classification prompts.

          page_range: The page range to process (1-indexed). By default, the first 5 pages are used.
              If more than 25 pages are selected, only the first 25 (after sorting) are used.
              Only applies to PDFs; ignored for other document types.

          persist_results: If True, persist the results indefinitely. Defaults to False.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/classify",
            body=maybe_transform(
                {
                    "input": input,
                    "classification_schema": classification_schema,
                    "document_metadata": document_metadata,
                    "page_range": page_range,
                    "persist_results": persist_results,
                },
                classify_run_params.ClassifyRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClassifyResponse,
        )


class AsyncClassifyResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClassifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncClassifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClassifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reductoai/reducto-python-sdk#with_streaming_response
        """
        return AsyncClassifyResourceWithStreamingResponse(self)

    async def run(
        self,
        *,
        input: classify_run_params.Input,
        classification_schema: Iterable[classify_run_params.ClassificationSchema] | Omit = omit,
        document_metadata: Optional[str] | Omit = omit,
        page_range: Optional[classify_run_params.PageRange] | Omit = omit,
        persist_results: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClassifyResponse:
        """
        Classify

        Args:
          input: For parse/split/extract pipelines, the URL of the document to be processed. You
              can provide one of the following: 1. A publicly available URL 2. A presigned S3
              URL 3. A reducto:// prefixed URL obtained from the /upload endpoint after
              directly uploading a document 4. A jobid:// prefixed URL obtained from a
              previous /parse invocation 5. A list of URLs (for multi-document pipelines, V3
              API only)

                          For edit pipelines, this should be a string containing the edit instructions

          classification_schema: A list of classification categories and their matching criteria.

          document_metadata: Optional document-level metadata to include in classification prompts.

          page_range: The page range to process (1-indexed). By default, the first 5 pages are used.
              If more than 25 pages are selected, only the first 25 (after sorting) are used.
              Only applies to PDFs; ignored for other document types.

          persist_results: If True, persist the results indefinitely. Defaults to False.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/classify",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "classification_schema": classification_schema,
                    "document_metadata": document_metadata,
                    "page_range": page_range,
                    "persist_results": persist_results,
                },
                classify_run_params.ClassifyRunParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClassifyResponse,
        )


class ClassifyResourceWithRawResponse:
    def __init__(self, classify: ClassifyResource) -> None:
        self._classify = classify

        self.run = to_raw_response_wrapper(
            classify.run,
        )


class AsyncClassifyResourceWithRawResponse:
    def __init__(self, classify: AsyncClassifyResource) -> None:
        self._classify = classify

        self.run = async_to_raw_response_wrapper(
            classify.run,
        )


class ClassifyResourceWithStreamingResponse:
    def __init__(self, classify: ClassifyResource) -> None:
        self._classify = classify

        self.run = to_streamed_response_wrapper(
            classify.run,
        )


class AsyncClassifyResourceWithStreamingResponse:
    def __init__(self, classify: AsyncClassifyResource) -> None:
        self._classify = classify

        self.run = async_to_streamed_response_wrapper(
            classify.run,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reducto import Reducto, AsyncReducto
from tests.utils import assert_matches_type
from reducto.types import ClassifyResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestClassify:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run(self, client: Reducto) -> None:
        classify = client.classify.run(
            input="string",
        )
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params(self, client: Reducto) -> None:
        classify = client.classify.run(
            input="string",
            classification_schema=[
                {
                    "category": "category",
                    "criteria": ["string"],
                }
            ],
            document_metadata="document_metadata",
            page_range={
                "end": 0,
                "start": 0,
            },
            persist_results=True,
        )
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run(self, client: Reducto) -> None:
        response = client.classify.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        classify = response.parse()
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run(self, client: Reducto) -> None:
        with client.classify.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            classify = response.parse()
            assert_matches_type(ClassifyResponse, classify, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncClassify:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run(self, async_client: AsyncReducto) -> None:
        classify = await async_client.classify.run(
            input="string",
        )
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params(self, async_client: AsyncReducto) -> None:
        classify = await async_client.classify.run(
            input="string",
            classification_schema=[
                {
                    "category": "category",
                    "criteria": ["string"],
                }
            ],
            document_metadata="document_metadata",
            page_range={
                "end": 0,
                "start": 0,
            },
            persist_results=True,
        )
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncReducto) -> None:
        response = await async_client.classify.with_raw_response.run(
            input="string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        classify = await response.parse()
        assert_matches_type(ClassifyResponse, classify, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run(self, async_client: AsyncReducto) -> None:
        async with async_client.classify.with_streaming_response.run(
            input="string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            classify = await response.parse()
            assert_matches_type(ClassifyResponse, classify, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel
from .bounding_box import BoundingBox

__all__ = ["EditWidget"]


class EditWidget(BaseModel):
    bbox: BoundingBox
    """Bounding box coordinates of the widget"""

    description: str
    """Description of the widget extracted from the document"""

    type: Literal["text", "checkbox", "radio", "dropdown", "barcode"]
    """Type of the form widget"""

    fill: Optional[bool] = None
    """If True (default), the system will attempt to fill this widget.

    If False, the widget will be created but intentionally left unfilled.
    """

    font_size: Optional[float] = None
    """Font size in points for this specific field.

    Takes priority over the global font_size in EditOptions. If not set, falls back
    to the global font_size, then to auto-calculated sizing.
    """

    value: Optional[str] = None
    """
    If provided, this value will be used directly instead of attempting to
    intelligently determine the field value.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import TypeAlias, TypedDict

from .svix_webhook_config import SvixWebhookConfig
from .direct_webhook_config import DirectWebhookConfig

__all__ = ["ConfigV3AsyncConfig", "Webhook"]

Webhook: TypeAlias = Union[SvixWebhookConfig, DirectWebhookConfig]


class ConfigV3AsyncConfig(TypedDict, total=False):
    metadata: object
    """JSON metadata included in webhook request body. Defaults to None."""

    priority: bool
    """
    If True, attempts to process the job with priority if the user has priority
    processing budget available; by default, sync jobs are prioritized above async
    jobs.
    """

    webhook: Optional[Webhook]
    """The webhook configuration for the asynchronous processing."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .split_large_tables import SplitLargeTables

__all__ = ["Spreadsheet"]


class Spreadsheet(BaseModel):
    clustering: Optional[Literal["accurate", "fast", "disabled"]] = None
    """
    In a spreadsheet with different tables inside, we enable splitting up the tables
    by default. Accurate mode applies more powerful models for superior accuracy, at
    5× the default per-cell rate. Disabling will register as one large table.
    """

    exclude: Optional[List[Literal["hidden_sheets", "hidden_rows", "hidden_cols", "styling", "spreadsheet_images"]]] = (
        None
    )
    """Whether to exclude hidden sheets, rows, or columns in the output."""

    include: Optional[List[Literal["cell_colors", "formula", "dropdowns"]]] = None
    """Whether to include cell color, formula, and dropdown information in the output."""

    split_large_tables: Optional[SplitLargeTables] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .enrich_config import EnrichConfig

__all__ = ["ExperimentalProcessingOptions"]


class ExperimentalProcessingOptions(BaseModel):
    chunk_table_blocks: Optional[bool] = None
    """
    If True, split table blocks into smaller chunks based on the specified chunk
    size in the chunking option. Defaults to False.
    """

    danger_filter_wide_boxes: Optional[bool] = None
    """You probably shouldn't use this.

    If True, filter out boxes with width greater than 50% of the document width.
    Defaults to False. You probably don't want to use this.
    """

    detect_signatures: Optional[bool] = None
    """If True, detect signatures in the document. Defaults to False."""

    embed_text_metadata_pdf: Optional[bool] = None
    """
    If extracted OCR text metadata should be embedded back into the returned PDF,
    overwriting any existing text. Defaults to False.
    """

    enable_checkboxes: Optional[bool] = None
    """
    Use an experimental checkbox detection model to add checkboxes to the output,
    defaults to False
    """

    enable_equations: Optional[bool] = None
    """
    Use an experimental equation detection model to add equations to the output,
    defaults to False
    """

    enable_scripts: Optional[bool] = None
    """
    Add <sub> tag around subscripts and <sup> tag around superscripts, defaults to
    False
    """

    enrich: Optional[EnrichConfig] = None
    """The configuration options for enrichment."""

    latency_sensitive: Optional[bool] = None
    """If True, the job will be processed with lower latency and higher priority.

    Uses 2x the cost of a regular job. Defaults to False.
    """

    layout_enrichment: Optional[bool] = None
    """
    Layout enrichment is a beta feature that improves our layout and reading order
    performance at the cost of increased latency. Defaults to False.
    """

    layout_model: Optional[Literal["default", "beta", "rfdetr"]] = None
    """The layout model to use for the document.

    This will be deprecated in the future.
    """

    native_office_conversion: Optional[bool] = None
    """
    Instead of using LibreOffice, when enabled, this flag uses a Windows VM to
    convert files. This is slower but more accurate.
    """

    promptable_agentic_text_on_regular_blocks: Optional[bool] = None
    """
    If True, enable two-stage LLM pipeline for agentic text correction on regular
    text blocks. Defaults to False.
    """

    return_figure_images: Optional[bool] = None
    """If figure images should be returned in the result. Defaults to False."""

    return_table_images: Optional[bool] = None
    """If table images should be returned in the result. Defaults to False."""

    rotate_figures: Optional[bool] = None
    """
    Use an orientation model to detect and rotate figures as needed, defaults to
    False
    """

    rotate_pages: Optional[bool] = None
    """Use an orientation model to detect and rotate pages as needed, defaults to True"""

    user_specified_timeout_seconds: Optional[float] = None
    """A user specified timeout, defaults to None"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .extract_usage import ExtractUsage

__all__ = ["ExtractResponse"]


class ExtractResponse(BaseModel):
    citations: Optional[List[object]] = None
    """The citations corresponding to the extracted response."""

    result: List[object]
    """The extracted response in your provided schema.

    This is a list of dictionaries. If disable_chunking is True (default), then it
    will be a list of length one.
    """

    usage: ExtractUsage

    job_id: Optional[str] = None

    studio_link: Optional[str] = None
    """The link to the studio pipeline for the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SplitCategory"]


class SplitCategory(BaseModel):
    description: str

    name: str

    partition_key: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DeepSplitPageEvidence"]


class DeepSplitPageEvidence(BaseModel):
    evidence: str

    page_number: int

    confidence: Optional[Literal["high", "medium", "low"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Upload"]


class Upload(BaseModel):
    file_id: str

    presigned_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .edit_widget import EditWidget
from .parse_usage import ParseUsage

__all__ = ["EditResponse"]


class EditResponse(BaseModel):
    document_url: str
    """Presigned URL to download the edited document."""

    form_schema: Optional[List[EditWidget]] = None
    """Form schema for PDF forms.

    List of widgets with their types, descriptions, and bounding boxes.
    """

    usage: Optional[ParseUsage] = None
    """
    Usage information for the edit operation, including number of pages and credits
    charged.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel
from .parse_usage import ParseUsage
from .deep_split_page_evidence import DeepSplitPageEvidence

__all__ = [
    "SplitResponse",
    "Result",
    "ResultSplitResult",
    "ResultSplitResultSplit",
    "ResultSplitResultSplitPartition",
    "ResultDeepSplitResult",
    "ResultDeepSplitResultSplit",
    "ResultDeepSplitResultSplitPartition",
]


class ResultSplitResultSplitPartition(BaseModel):
    name: str

    pages: List[int]

    conf: Optional[Literal["high", "low"]] = None


class ResultSplitResultSplit(BaseModel):
    name: str

    pages: List[int]

    conf: Optional[Literal["high", "low"]] = None

    partitions: Optional[List[ResultSplitResultSplitPartition]] = None


class ResultSplitResult(BaseModel):
    section_mapping: Optional[Dict[str, List[int]]] = None

    splits: List[ResultSplitResultSplit]


class ResultDeepSplitResultSplitPartition(BaseModel):
    name: str

    pages: List[DeepSplitPageEvidence]


class ResultDeepSplitResultSplit(BaseModel):
    name: str

    pages: List[DeepSplitPageEvidence]

    partitions: Optional[List[ResultDeepSplitResultSplitPartition]] = None


class ResultDeepSplitResult(BaseModel):
    splits: List[ResultDeepSplitResultSplit]


Result: TypeAlias = Union[ResultSplitResult, ResultDeepSplitResult]


class SplitResponse(BaseModel):
    result: Result
    """The split result."""

    usage: ParseUsage



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["DirectWebhookConfig"]


class DirectWebhookConfig(BaseModel):
    url: str

    mode: Optional[Literal["direct"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["Chunking"]


class Chunking(TypedDict, total=False):
    chunk_mode: Literal["variable", "section", "page", "disabled", "block", "page_sections"]
    """Choose how to partition chunks.

    Variable mode chunks by character length and visual context. Section mode chunks
    by section headers. Page mode chunks according to pages. Page sections mode
    chunks first by page, then by sections within each page. Disabled returns one
    single chunk.
    """

    chunk_overlap: int
    """Number of characters of overlap to include from adjacent chunks. Defaults to 0."""

    chunk_size: Optional[int]
    """
    The approximate size of chunks (in characters) that the document will be split
    into. Defaults to null, in which case the chunk size is variable between 250 -
    1500 characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reducto import Reducto, AsyncReducto
from tests.utils import assert_matches_type
from reducto.types import EditResponse, EditRunJobResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEdit:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run(self, client: Reducto) -> None:
        edit = client.edit.run(
            document_url="string",
            edit_instructions="edit_instructions",
        )
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_with_all_params(self, client: Reducto) -> None:
        edit = client.edit.run(
            document_url="string",
            edit_instructions="edit_instructions",
            edit_options={
                "color": "#e1cb97",
                "enable_overflow_pages": True,
                "flatten": True,
                "font_size": 1,
                "llm_provider_preference": "openai",
            },
            form_schema=[
                {
                    "bbox": {
                        "height": 0,
                        "left": 0,
                        "page": 0,
                        "top": 0,
                        "width": 0,
                        "original_page": 0,
                    },
                    "description": "description",
                    "type": "text",
                    "fill": True,
                    "font_size": 1,
                    "value": "value",
                }
            ],
            priority=True,
        )
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run(self, client: Reducto) -> None:
        response = client.edit.with_raw_response.run(
            document_url="string",
            edit_instructions="edit_instructions",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        edit = response.parse()
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run(self, client: Reducto) -> None:
        with client.edit.with_streaming_response.run(
            document_url="string",
            edit_instructions="edit_instructions",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            edit = response.parse()
            assert_matches_type(EditResponse, edit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job(self, client: Reducto) -> None:
        edit = client.edit.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        )
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_run_job_with_all_params(self, client: Reducto) -> None:
        edit = client.edit.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
            edit_options={
                "color": "#e1cb97",
                "enable_overflow_pages": True,
                "flatten": True,
                "font_size": 1,
                "llm_provider_preference": "openai",
            },
            form_schema=[
                {
                    "bbox": {
                        "height": 0,
                        "left": 0,
                        "page": 0,
                        "top": 0,
                        "width": 0,
                        "original_page": 0,
                    },
                    "description": "description",
                    "type": "text",
                    "fill": True,
                    "font_size": 1,
                    "value": "value",
                }
            ],
            priority=True,
            webhook={
                "channels": ["string"],
                "metadata": {},
                "mode": "disabled",
                "url": "url",
            },
        )
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_run_job(self, client: Reducto) -> None:
        response = client.edit.with_raw_response.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        edit = response.parse()
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_run_job(self, client: Reducto) -> None:
        with client.edit.with_streaming_response.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            edit = response.parse()
            assert_matches_type(EditRunJobResponse, edit, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncEdit:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run(self, async_client: AsyncReducto) -> None:
        edit = await async_client.edit.run(
            document_url="string",
            edit_instructions="edit_instructions",
        )
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_with_all_params(self, async_client: AsyncReducto) -> None:
        edit = await async_client.edit.run(
            document_url="string",
            edit_instructions="edit_instructions",
            edit_options={
                "color": "#e1cb97",
                "enable_overflow_pages": True,
                "flatten": True,
                "font_size": 1,
                "llm_provider_preference": "openai",
            },
            form_schema=[
                {
                    "bbox": {
                        "height": 0,
                        "left": 0,
                        "page": 0,
                        "top": 0,
                        "width": 0,
                        "original_page": 0,
                    },
                    "description": "description",
                    "type": "text",
                    "fill": True,
                    "font_size": 1,
                    "value": "value",
                }
            ],
            priority=True,
        )
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run(self, async_client: AsyncReducto) -> None:
        response = await async_client.edit.with_raw_response.run(
            document_url="string",
            edit_instructions="edit_instructions",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        edit = await response.parse()
        assert_matches_type(EditResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run(self, async_client: AsyncReducto) -> None:
        async with async_client.edit.with_streaming_response.run(
            document_url="string",
            edit_instructions="edit_instructions",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            edit = await response.parse()
            assert_matches_type(EditResponse, edit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job(self, async_client: AsyncReducto) -> None:
        edit = await async_client.edit.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        )
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_run_job_with_all_params(self, async_client: AsyncReducto) -> None:
        edit = await async_client.edit.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
            edit_options={
                "color": "#e1cb97",
                "enable_overflow_pages": True,
                "flatten": True,
                "font_size": 1,
                "llm_provider_preference": "openai",
            },
            form_schema=[
                {
                    "bbox": {
                        "height": 0,
                        "left": 0,
                        "page": 0,
                        "top": 0,
                        "width": 0,
                        "original_page": 0,
                    },
                    "description": "description",
                    "type": "text",
                    "fill": True,
                    "font_size": 1,
                    "value": "value",
                }
            ],
            priority=True,
            webhook={
                "channels": ["string"],
                "metadata": {},
                "mode": "disabled",
                "url": "url",
            },
        )
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_run_job(self, async_client: AsyncReducto) -> None:
        response = await async_client.edit.with_raw_response.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        edit = await response.parse()
        assert_matches_type(EditRunJobResponse, edit, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_run_job(self, async_client: AsyncReducto) -> None:
        async with async_client.edit.with_streaming_response.run_job(
            document_url="string",
            edit_instructions="edit_instructions",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            edit = await response.parse()
            assert_matches_type(EditRunJobResponse, edit, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["FigureAgentic"]


class FigureAgentic(BaseModel):
    scope: Literal["figure"]

    advanced_chart_agent: Optional[bool] = None
    """If True, use the advanced chart agent. Defaults to False."""

    prompt: Optional[str] = None
    """Custom prompt for figure agentic."""

    return_overlays: Optional[bool] = None
    """If True, return overlays for the figure.

    This is so you can use the overlays to double check the quality of the
    extraction
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["PipelineSettingsParam"]


class PipelineSettingsParam(TypedDict, total=False):
    """Settings for pipeline execution that override pipeline defaults."""

    document_password: Optional[str]
    """Password to decrypt password-protected documents."""
